!+ adjoint of calculation of optical depths
!
SUBROUTINE OPDEPAD &
     (knpf, kchan, kprof, knchpf, ksat,            &
     opdp, xxm, xxw, xxo, xxc, debyeprof ,         &
     opdp_d, opdpref_d, zopdpref_d, xxc_d, debyeprof_d )
  !
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  !
  ! Description:
  !  AD of routine
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  !
  ! Method:
  !          for gases: ECMWF TECH MEM 345.
  !          for cloud liquid water: see comments in the code
  !   
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date        Comment
  ! -------   ----        -------
  !          04/03/92.    Change to +ve opt dep and add traps
  !          04/09/92.    For version 2. move io to arguments, add sat index,
  !                         change inner loops (faster).
  !            7/7/97     Added ozone, extended water vapour 
  !           15/8/97     Correct bug in adjoint of OPDEP (Erik)
  !            6/7/98     Improved vectorisation of routines (Roger)
  !            7/9/99     Added LWP + SSM/I logic (English/Saunders)
  !        01/05/2000     F90 code 
  !        23/03/2001     New coef. format, new channel numbers (P. Brunel)
  !        09/08/2001     New predictors (Marco Matricardi)
  !        14/09/2001     Changed to use structures.  Andrew Collard.
  !        11/03/2004     Added test on layer optical depth. Andrew Collard.
  !
  ! Code Description:
  !   Language:          Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  !
  USE MOD_CPARAM, ONLY : &
  ! Imported Scalar Variables with intent (in):
  jplev   ,&    ! No. of pressure levels
  njplev  ,&    ! No. of pressure levels
  njpcofm ,&    ! Mixed gas coeffs (max
  njpcofw ,&    ! Water vapour coeffs (max
  njpcofo ,&    ! Ozone coeffs
  nmwcldtop     ! Upper level for lwp calcs

  Use MOD_FMV, Only : &
  ! imported arrays with intent (out):
  sensor ,&     ! sensor number for each triplet
  fmv_gas       ! number of active gases

  USE MOD_TAUCFN, ONLY : &
       ! imported structure with intent (out):
       TransCoeffs
         ! Structure contains:   
         !    cfm  :  mixed gas coeffs
         !    cfw  :  water vapour coeffs 
         !    cfo  :  ozone coeffs

  Use MOD_TOVCHN, Only : &
       ! Imported structure with intent (in):
       ChanConstants
         ! Structure contains:   
         !    freq   :   channel frequencies in ghz
         !    gamma  :   "gamma factor" transmittance corrections

  IMPLICIT NONE
  
  ! Subroutine arguments
  ! Scalar arguments with intent(in):
  INTEGER , INTENT(in) :: knpf   ! Number of profiles
  INTEGER , INTENT(in) :: knchpf ! Number of radiances (chans * profs).
  INTEGER , INTENT(in) :: ksat   ! Satellite index.

  ! Array  arguments with intent(in):
  INTEGER , INTENT(in) :: kchan(knchpf) ! Channel indices for each rad
  INTEGER , INTENT(in) :: kprof(knchpf) ! Profile indices for each rad
  ! Forward model
  REAL, INTENT(in) :: xxc_d(njplev,knpf,1)       ! ..... for cloud liquid water
  REAL, INTENT(in) :: debyeprof_d(5,njplev,knpf) ! Functions of debye terms of 
                                                 !   profile for cloud calc
  REAL, INTENT(inout) :: opdp_d(knchpf,njplev)   ! Optical depth from press 
                                                 !    level to space
  REAL, INTENT(in) :: opdpref_d(knchpf,njplev)   ! same as OPDP but without 
                                                 !   control check
  REAL, INTENT(in) :: zopdpref_d(njplev,knchpf)  ! ZOPDP before sign check  

  ! Array  arguments with intent(inout):
  ! adjoint
  REAL, INTENT(inout) :: opdp(knchpf,njplev)    ! Optical depth from press 
                                              !    level to space
  REAL, INTENT(inout) :: xxm(njplev,knpf,njpcofm) ! Func. of profile for MxG
  REAL, INTENT(inout) :: xxw(njplev,knpf,njpcofw) ! .... for water vapour
  REAL, INTENT(inout) :: xxo(njplev,knpf,njpcofo) ! .... for ozone 
  REAL, INTENT(inout) :: xxc(njplev,knpf,1)       ! .... for cloud liquid water
  REAL, INTENT(inout) :: debyeprof(5,njplev,knpf) ! Functions of debye terms of 
  ! Local scalars:
  INTEGER :: j
  INTEGER :: jl
  INTEGER :: ich
  INTEGER :: ipf

  ! forward model
  REAL  :: zdivsqr_d
  REAL  :: z34dif_d, z45dif_d, z1sq_d, z2sq_d, z1div_d, z2div_d
  REAL  :: z1den_d, z2den_d, zastar_d, z1prod_d, z2prod_d, z3prod_d, z4prod_d
  REAL  :: zbstar_d, zbstarsq_d, za2star_d, za2starsq_d, zdiv_d, zgstar_d
  REAL  :: z1fsqz1sq_d, z2fsqz2sq_d

  ! adjoint
  REAL  :: zcl
  REAL  :: zf, zfsq, z34dif, z45dif, z1sq, z2sq, z1div, z2div
  REAL  :: z1den, z2den, zastar, z1prod, z2prod, z3prod, z4prod
  REAL  :: zbstar, zbstarsq, za2star, za2starsq, zdiv, zgstar
  REAL  :: z1fsqz1sq, z2fsqz2sq

  ! Local arrays:
  REAL  :: opdpa(knchpf)      ! Workspace used in calculating opdp
  REAL  :: zopdp(njplev,knchpf)
  !- End of header ------------------------------------------------------
  !
  opdpa(:)    = 0.0
  !
  DO jl = jplev, 1, -1
     DO j = 1, knchpf
        ich = kchan(j)
        !         
        ! 4.   Check for reasonable value
        ! ----- --- ---------- -----
        IF(jl == 1) THEN
           IF (opdpref_d(j,jl) < 0.) opdp(j,jl)=0.
        ELSE
           IF (opdpref_d(j,jl) < opdp_d(j,jl)) THEN
              opdp(j,jl-1) = opdp(j,jl-1)+opdp(j,jl)
              opdp(j,jl)   = 0.
           ENDIF
        ENDIF

        ! Reset opdep_d in case it is going to be used again
        ! -------------------------------------------------
        opdp_d(j,jl) = opdpref_d(j,jl)
        !
        !* 3.   Apply gamma corrections and change to +ve opt depth.
        ! ----- ----- ----------- --- ------ -- --- --- ------
        opdp(j,jl)  = -opdp(j,jl)*ChanConstants(ksat) % gamma(ich)
        opdpa(j)    = opdpa(j) + opdp(j,jl)
        zopdp(jl,j) = opdpa(j)
     ENDDO
  ENDDO

  where( zopdpref_d(:,:) > 0.0 )
     zopdp = 0.0
  end where

  !
  ! -----------------------------------------------------------------
  !* 1.   Calculate gaseous optical depths (* -1).
  ! --------- ------- ------- ------ -------
  DO j=1,knchpf
     ich = kchan(j)
     ipf = kprof(j)
     !
     !*  1.4  Cloud liquid water (microwave channels only)
     DO jl = njplev, 1, -1
        !
        IF (sensor(ksat) /=2 .OR. jl < nmwcldtop .OR. xxc_d(1,ipf,1) < 0.0) THEN
              zopdp(jl,j) = zopdp(jl,j) 
        ELSE
              ! Repeat direct code
              zf          = ChanConstants(ksat) % freq(ich)
              zfsq        = zf*zf
              z34dif_d    = debyeprof_d(3,jl,ipf) - debyeprof_d(4,jl,ipf)
              z45dif_d    = debyeprof_d(4,jl,ipf) - debyeprof_d(5,jl,ipf)
              z1sq_d      = debyeprof_d(1,jl,ipf)*debyeprof_d(1,jl,ipf)
              z2sq_d      = debyeprof_d(2,jl,ipf)*debyeprof_d(2,jl,ipf)
              z1fsqz1sq_d = zfsq + z1sq_d
              z2fsqz2sq_d = zfsq + z2sq_d
              z1div_d     = 1.0/z1fsqz1sq_d
              z2div_d     = 1.0/z2fsqz2sq_d
              z1den_d     = z34dif_d*z1div_d
              z2den_d     = z45dif_d*z2div_d
              zastar_d    = debyeprof_d(3,jl,ipf) - zfsq*(z1den_d + z2den_d)
              z1prod_d    = z34dif_d*debyeprof_d(1,jl,ipf)
              z2prod_d    = z1prod_d*z1div_d
              z3prod_d    = z45dif_d*debyeprof_d(2,jl,ipf)
              z4prod_d    = z3prod_d*z2div_d
              zbstar_d    = -zf*(z2prod_d + z4prod_d)
              zbstarsq_d  = zbstar_d*zbstar_d
              za2star_d   = zastar_d + 2.0
              za2starsq_d = za2star_d*za2star_d
              zdiv_d      = za2starsq_d + zbstarsq_d
              zgstar_d    = -3.0*zbstar_d/zdiv_d
              ! Now do Adjoint code            
              zcl         = -zopdp(jl,j)
              zgstar      = 1.5*zf*zcl*xxc_d(jl,ipf,1)
              zdivsqr_d   = 1.0/(zdiv_d*zdiv_d)
              zdiv        = 3.0*(zgstar*zbstar_d)*zdivsqr_d
              zbstar      = -3.0*(zdiv_d*zgstar)*zdivsqr_d
              za2starsq   = zdiv
              zbstarsq    = zdiv
              za2star     = 2.0*za2star_d*za2starsq
              zastar      = za2star
              zbstar      = zbstar + 2.0*zbstar_d*zbstarsq
              z4prod      = -zf*zbstar
              z2prod      = -zf*zbstar
              z3prod      = z2div_d*z4prod
              z2div       = z3prod_d*z4prod
              z45dif      = z3prod*debyeprof_d(2,jl,ipf)
              debyeprof(2,jl,ipf) = debyeprof(2,jl,ipf) + z45dif_d*z3prod
              z1prod      = z1div_d*z2prod
              z1div       = z1prod_d*z2prod
              z34dif      = z1prod*debyeprof_d(1,jl,ipf)
              debyeprof(1,jl,ipf) = debyeprof(1,jl,ipf) + z34dif_d*z1prod
              debyeprof(3,jl,ipf) = debyeprof(3,jl,ipf) + zastar
              z1den       = -zfsq*zastar
              z2den       = -zfsq*zastar
              z2div       = z2div + z45dif_d*z2den
              z45dif      = z45dif + z2div_d*z2den
              z1div       = z1div + z34dif_d*z1den
              z34dif      = z34dif + z1div_d*z1den
              z2fsqz2sq   = -z2div/(z2fsqz2sq_d*z2fsqz2sq_d)
              z1fsqz1sq   = -z1div/(z1fsqz1sq_d*z1fsqz1sq_d)
              z2sq        = z2fsqz2sq           
              z1sq        = z1fsqz1sq 
              debyeprof(2,jl,ipf) = debyeprof(2,jl,ipf)              &
                                    + 2.0*debyeprof_d(2,jl,ipf)*z2sq        
              debyeprof(1,jl,ipf) = debyeprof(1,jl,ipf)              &
                                    + 2.0*debyeprof_d(1,jl,ipf)*z1sq
              debyeprof(4,jl,ipf) = debyeprof(4,jl,ipf) + z45dif
              debyeprof(5,jl,ipf) = debyeprof(5,jl,ipf) - z45dif
              debyeprof(3,jl,ipf) = debyeprof(3,jl,ipf) + z34dif
              debyeprof(4,jl,ipf) = debyeprof(4,jl,ipf) - z34dif
              xxc(jl,ipf,1) = xxc(jl,ipf,1) + 1.5*zf*zgstar_d*zcl
        ENDIF
     ENDDO
     !               
     !* 1.3  Ozone (for HIRS channels only)
     !
     ! New: ozone ==> if fmv_gas(ksat) > 2 
     ! can be replaced in the future by a flag on each channel
     ! indicating the absorption gases.
     IF( fmv_gas(ksat) > 2 ) THEN
        ! All ozone channels: use predictors from Matricardi 2000
        ! For TL terms with SQQO as the linearized variable 
        DO jl = njplev, 1, -1
            XXO(JL,IPF,11)=XXO(JL,IPF,11)+TransCoeffs(ksat) % CFO(JL,ICH,11) *ZOPDP(JL,J)
            XXO(JL,IPF,10)=XXO(JL,IPF,10)+TransCoeffs(ksat) % CFO(JL,ICH,10) *ZOPDP(JL,J)
            XXO(JL,IPF,9) =XXO(JL,IPF,9) +TransCoeffs(ksat) % CFO(JL,ICH,9)  *ZOPDP(JL,J)
            XXO(JL,IPF,8) =XXO(JL,IPF,8) +TransCoeffs(ksat) % CFO(JL,ICH,8)  *ZOPDP(JL,J)
            XXO(JL,IPF,7) =XXO(JL,IPF,7) +TransCoeffs(ksat) % CFO(JL,ICH,7)  *ZOPDP(JL,J)
            XXO(JL,IPF,6) =XXO(JL,IPF,6) +TransCoeffs(ksat) % CFO(JL,ICH,6)  *ZOPDP(JL,J)
            XXO(JL,IPF,5) =XXO(JL,IPF,5) +TransCoeffs(ksat) % CFO(JL,ICH,5)  *ZOPDP(JL,J)
            XXO(JL,IPF,4) =XXO(JL,IPF,4) +TransCoeffs(ksat) % CFO(JL,ICH,4)  *ZOPDP(JL,J)
            XXO(JL,IPF,3) =XXO(JL,IPF,3) +TransCoeffs(ksat) % CFO(JL,ICH,3)  *ZOPDP(JL,J)
            XXO(JL,IPF,2) =XXO(JL,IPF,2) +TransCoeffs(ksat) % CFO(JL,ICH,2)  *ZOPDP(JL,J)
            XXO(JL,IPF,1) =XXO(JL,IPF,1) +TransCoeffs(ksat) % CFO(JL,ICH,1)  *ZOPDP(JL,J)
        ENDDO
     ENDIF
     !
     !* 1.2  Water vapour
     !
     ! All channels use predictors from Matricardi 2000
        ! For TL terms with SQQ as the linearized variable
     DO jl = njplev, 1, -1
          XXW(JL,IPF,15)=XXW(JL,IPF,15)+TransCoeffs(ksat) % CFW(JL,ICH,15)*ZOPDP(JL,J)
          XXW(JL,IPF,14)=XXW(JL,IPF,14)+TransCoeffs(ksat) % CFW(JL,ICH,14)*ZOPDP(JL,J)
          XXW(JL,IPF,13)=XXW(JL,IPF,13)+TransCoeffs(ksat) % CFW(JL,ICH,13)*ZOPDP(JL,J)
          XXW(JL,IPF,12)=XXW(JL,IPF,12)+TransCoeffs(ksat) % CFW(JL,ICH,12)*ZOPDP(JL,J)
          XXW(JL,IPF,11)=XXW(JL,IPF,11)+TransCoeffs(ksat) % CFW(JL,ICH,11)*ZOPDP(JL,J)
          XXW(JL,IPF,10)=XXW(JL,IPF,10)+TransCoeffs(ksat) % CFW(JL,ICH,10)*ZOPDP(JL,J)
          XXW(JL,IPF,9) =XXW(JL,IPF,9)+TransCoeffs(ksat) % CFW(JL,ICH,9)*ZOPDP(JL,J)
          XXW(JL,IPF,8) =XXW(JL,IPF,8)+TransCoeffs(ksat) % CFW(JL,ICH,8)*ZOPDP(JL,J)
          XXW(JL,IPF,7) =XXW(JL,IPF,7)+TransCoeffs(ksat) % CFW(JL,ICH,7)*ZOPDP(JL,J)
          XXW(JL,IPF,6) =XXW(JL,IPF,6)+TransCoeffs(ksat) % CFW(JL,ICH,6)*ZOPDP(JL,J)
          XXW(JL,IPF,5) =XXW(JL,IPF,5)+TransCoeffs(ksat) % CFW(JL,ICH,5)*ZOPDP(JL,J)
          XXW(JL,IPF,4) =XXW(JL,IPF,4)+TransCoeffs(ksat) % CFW(JL,ICH,4)*ZOPDP(JL,J)
          XXW(JL,IPF,3) =XXW(JL,IPF,3)+TransCoeffs(ksat) % CFW(JL,ICH,3)*ZOPDP(JL,J)
          XXW(JL,IPF,2) =XXW(JL,IPF,2)+TransCoeffs(ksat) % CFW(JL,ICH,2)*ZOPDP(JL,J)
          XXW(JL,IPF,1) =XXW(JL,IPF,1)+TransCoeffs(ksat) % CFW(JL,ICH,1)*ZOPDP(JL,J)
     ENDDO
     !
     !* 1.1  Uniformly mixed gases
     !
     ! all channels: use predictors from Matricardi 2000
     ! All TL terms have the XXMs as the linearized variables
     DO jl = njplev,1,-1
          XXM(JL,IPF,10)=XXM(JL,IPF,10)+TransCoeffs(ksat) % CFM(JL,ICH,10)*ZOPDP(JL,J)
          XXM(JL,IPF,9) =XXM(JL,IPF,9)+TransCoeffs(ksat) % CFM(JL,ICH,9)*ZOPDP(JL,J)
          XXM(JL,IPF,8) =XXM(JL,IPF,8)+TransCoeffs(ksat) % CFM(JL,ICH,8)*ZOPDP(JL,J)
          XXM(JL,IPF,7) =XXM(JL,IPF,7)+TransCoeffs(ksat) % CFM(JL,ICH,7)*ZOPDP(JL,J)
          XXM(JL,IPF,6) =XXM(JL,IPF,6)+TransCoeffs(ksat) % CFM(JL,ICH,6)*ZOPDP(JL,J)
          XXM(JL,IPF,5) =XXM(JL,IPF,5)+TransCoeffs(ksat) % CFM(JL,ICH,5)*ZOPDP(JL,J)
          XXM(JL,IPF,4) =XXM(JL,IPF,4)+TransCoeffs(ksat) % CFM(JL,ICH,4)*ZOPDP(JL,J)
          XXM(JL,IPF,3) =XXM(JL,IPF,3)+TransCoeffs(ksat) % CFM(JL,ICH,3)*ZOPDP(JL,J)
          XXM(JL,IPF,2) =XXM(JL,IPF,2)+TransCoeffs(ksat) % CFM(JL,ICH,2)*ZOPDP(JL,J)
          XXM(JL,IPF,1) =XXM(JL,IPF,1)+TransCoeffs(ksat) % CFM(JL,ICH,1)*ZOPDP(JL,J)
     ENDDO
  ENDDO
  RETURN
END SUBROUTINE OPDEPAD
