!+ Tangent linear of calculation of optical depths
!
SUBROUTINE OPDEPTL &
     (knpf, kchan, kprof, knchpf, ksat, opdp, xxm, xxw, xxo, xxc, &
     debyeprof, opdp_d, opdpref_d, zopdpref_d, xxc_d, debyeprof_d )
  !
  !    This software was developed within the context of 
  !    the EUMETSAT Satellite Application Facility on 
  !    Numerical Weather Prediction (NWP SAF), under the 
  !    Cooperation Agreement dated 25 November 1998, between 
  !    EUMETSAT and the Met Office, UK, by one or more partners 
  !    within the NWP SAF. The partners in the NWP SAF are 
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  ! 
  !    Copyright 2002, EUMETSAT, All Rights Reserved.
  !
  ! Description:
  !  TL of routine
  !  To calculate optical depths for a number of channels
  !  and profiles from every pressure level to space.
  !
  ! Method:
  !          for gases: ECMWF TECH MEM 345.
  !          for cloud liquid water: see comments in the code
  !   
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date        Comment
  ! -------   ----        -------
  !          04/03/92.    Change to +ve opt dep and add traps
  !          04/09/92.    For version 2. move io to arguments, add sat index,
  !                         change inner loops (faster).
  !            7/7/97     Added ozone, extended water vapour 
  !           15/8/97     Correct bug in adjoint of OPDEP (Erik)
  !            6/7/98     Improved vectorisation of routines (Roger)
  !            7/9/99     Added LWP + SSM/I logic (English/Saunders)
  !        01/05/2000     F90 code 
  !        23/03/2001     New coef. format, new channel numbers (P. Brunel)
  !        09/08/2001     New predictors (MArco Matricardi)
  !        14/09/2001     Changed to use structures.  Andrew Collard.
  !        11/03/2004     Added test on layer optical depth. Andrew Collard.
  !
  !
  ! Code Description:
  !   Language:          Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  !
  USE MOD_CPARAM, ONLY : &
  ! Imported Scalar Variables with intent (in):
  jplev   ,&    ! No. of pressure levels
  njplev  ,&    ! No. of pressure levels
  njpcofm ,&    ! Mixed gas coeffs (max
  njpcofw ,&    ! Water vapour coeffs (max
  njpcofo ,&    ! Ozone coeffs
  nmwcldtop     ! Upper level for lwp calcs

  Use MOD_FMV, Only : &
  ! imported arrays with intent (out):
  sensor ,&     ! sensor number for each triplet
  fmv_gas       ! number of active gases

  USE MOD_TAUCFN, ONLY : &
       ! imported structure with intent (out):
       TransCoeffs
         ! Structure contains:   
         !    cfm  :  mixed gas coeffs
         !    cfw  :  water vapour coeffs 
         !    cfo  :  ozone coeffs

  Use MOD_TOVCHN, Only : &
       ! Imported structure with intent (in):
       ChanConstants
         ! Structure contains:   
         !    freq   :   channel frequencies in ghz
         !    gamma  :   "gamma factor" transmittance corrections

  IMPLICIT NONE
  
  ! Subroutine arguments
  ! Scalar arguments with intent(in):
  INTEGER , INTENT(in) :: knpf   ! Number of profiles
  INTEGER , INTENT(in) :: knchpf ! Number of radiances (chans * profs).
  INTEGER , INTENT(in) :: ksat   ! Satellite index.

  ! Array  arguments with intent(in):
  INTEGER , INTENT(in) :: kchan(knchpf) ! Channel indices for each rad
  INTEGER , INTENT(in) :: kprof(knchpf) ! Profile indices for each rad
  ! Forward model
  REAL, INTENT(in) :: xxc_d(njplev,knpf,1)       ! ..... for cloud liquid water
  REAL, INTENT(in) :: debyeprof_d(5,njplev,knpf) ! Functions of debye terms of 
                                                 !   profile for cloud calc
  REAL, INTENT(in) :: opdp_d(knchpf,njplev)      ! Optical depth from press 
                                                 !    level to space
  REAL, INTENT(in) :: opdpref_d(knchpf,njplev)   ! same as OPDP but without 
                                                 !   control check
  REAL, INTENT(in) :: zopdpref_d(njplev,knchpf)  ! ZOPDP before sign check  
  ! Tangent linear
  REAL, INTENT(in) :: xxm(njplev,knpf,njpcofm) ! Func. of profile for mixed gas
  REAL, INTENT(in) :: xxw(njplev,knpf,njpcofw) ! ...... for water vapour
  REAL, INTENT(in) :: xxo(njplev,knpf,njpcofo) ! ...... for ozone 
  REAL, INTENT(in) :: xxc(njplev,knpf,1)       ! ...... for cloud liquid water
  REAL, INTENT(in) :: debyeprof(5,njplev,knpf) ! Functions of debye terms of 
                                               !   profile for cloud calc
  ! Array  arguments with intent(out):
  ! Tangent linear
  REAL, INTENT(out) :: opdp(knchpf,njplev)    ! Optical depth from press 
                                              !    level to space

  ! Local scalars:
  INTEGER :: j
  INTEGER :: jl
  INTEGER :: ich
  INTEGER :: ipf

  ! forward model
  REAL  :: z34dif_d, z45dif_d, z1sq_d, z2sq_d, z1div_d, z2div_d
  REAL  :: z1den_d, z2den_d, zastar_d, z1prod_d, z2prod_d, z3prod_d, z4prod_d
  REAL  :: zbstar_d, zbstarsq_d, za2star_d, za2starsq_d, zdiv_d, zgstar_d
  REAL  :: z1fsqz1sq_d, z2fsqz2sq_d

  REAL  :: zf, zfsq, z34dif, z45dif, z1sq, z2sq, z1div, z2div
  REAL  :: z1den, z2den, zastar, z1prod, z2prod, z3prod, z4prod
  REAL  :: zbstar, zbstarsq, za2star, za2starsq, zdiv, zgstar
  REAL  :: z1fsqz1sq, z2fsqz2sq
 
  ! Local arrays:
  REAL  :: opdpa(knchpf)      ! Workspace used in calculating opdp
  REAL  :: zopdp(njplev,knchpf)
  !- End of header ------------------------------------------------------
  !
  opdp(:,:)   = 0.0
  opdpa(:)    = 0.0
  !     -----------------------------------------------------------------
  !*         1.   Calculate layer gaseous optical depths (* -1).
  !               --------- ----- ------- ------- ------ -------
  DO j = 1, knchpf
     ich     = kchan(j)
     ipf     = kprof(j)
     !
     DO jl = 1, njplev
        !
        !*         1.1  Uniformly mixed gases
        !
        ! All channels: use predictors from Matricardi 2000 
        ! All terms have the XXMs as the linearized variables
        zopdp(jl,j)=                             &
             TransCoeffs(ksat) % cfm(jl,ich,1)*xxm(jl,ipf,1)+   &
             TransCoeffs(ksat) % cfm(jl,ich,2)*xxm(jl,ipf,2)+   &
             TransCoeffs(ksat) % cfm(jl,ich,3)*xxm(jl,ipf,3)+   &
             TransCoeffs(ksat) % cfm(jl,ich,4)*xxm(jl,ipf,4)+   &
             TransCoeffs(ksat) % cfm(jl,ich,5)*xxm(jl,ipf,5)+   &
             TransCoeffs(ksat) % cfm(jl,ich,6)*xxm(jl,ipf,6)+   &
             TransCoeffs(ksat) % cfm(jl,ich,7)*xxm(jl,ipf,7)+   &
             TransCoeffs(ksat) % cfm(jl,ich,8)*xxm(jl,ipf,8)+   &
             TransCoeffs(ksat) % cfm(jl,ich,9)*xxm(jl,ipf,9)+   &
             TransCoeffs(ksat) % cfm(jl,ich,10)*xxm(jl,ipf,10)
     ENDDO
        !
        !*         1.2  Water vapour
        ! All channels: use predictors from Matricardi 2000 
     DO jl = 1, njplev
        zopdp(jl,j)=zopdp(jl,j)+                   &
                 TransCoeffs(ksat) % cfw(jl,ich,1)*xxw(jl,ipf,1)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,2)*xxw(jl,ipf,2)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,3)*xxw(jl,ipf,3)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,4)*xxw(jl,ipf,4)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,5)*xxw(jl,ipf,5)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,6)*xxw(jl,ipf,6)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,7)*xxw(jl,ipf,7)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,8)*xxw(jl,ipf,8)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,9)*xxw(jl,ipf,9)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,10)*xxw(jl,ipf,10)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,11)*xxw(jl,ipf,11)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,12)*xxw(jl,ipf,12)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,13)*xxw(jl,ipf,13)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,14)*xxw(jl,ipf,14)+  &
                 TransCoeffs(ksat) % cfw(jl,ich,15)*xxw(jl,ipf,15)  
     ENDDO
     !*         1.3  Ozone  (only for IR channels)
     !
     ! New: ozone ==> if fmv_gas(ksat) > 2 
     ! can be replaced in the future by a flag on each channel
     ! indicating the absorption gases.
     IF( fmv_gas(ksat) > 2 ) THEN
        ! All ozone channels use predictors from Matricardi 2000 
        ! For terms with SQQO as the linearized variable
     DO jl = 1, njplev
           zopdp(jl,j)=zopdp(jl,j)+ &
                 TransCoeffs(ksat) % cfo(jl,ich,1)*xxo(jl,ipf,1)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,2)*xxo(jl,ipf,2)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,3)*xxo(jl,ipf,3)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,4)*xxo(jl,ipf,4)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,5)*xxo(jl,ipf,5)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,6)*xxo(jl,ipf,6)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,7)*xxo(jl,ipf,7)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,8)*xxo(jl,ipf,8)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,9)*xxo(jl,ipf,9)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,10)*xxo(jl,ipf,10)+   &
                 TransCoeffs(ksat) % cfo(jl,ich,11)*xxo(jl,ipf,11)
     ENDDO
     ENDIF
     !
     DO jl = 1, jplev
        !   
        !*         1.4   Cloud liquid water  (only for MW channels)
        !   
        IF(sensor(ksat) /=2 .OR. jl < nmwcldtop .OR. xxc_d(1,ipf,1) < 0.0)THEN
              zopdp(jl,j) = zopdp(jl,j)
        ELSE
              ! Repeat direct code  (see comments in opdep.f)

              zf       = ChanConstants(ksat) % freq(ich)
              zfsq     = zf*zf
              z34dif_d   = debyeprof_d(3,jl,ipf) - debyeprof_d(4,jl,ipf)
              z45dif_d   = debyeprof_d(4,jl,ipf) - debyeprof_d(5,jl,ipf)
              z1sq_d     = debyeprof_d(1,jl,ipf)*debyeprof_d(1,jl,ipf)
              z2sq_d     = debyeprof_d(2,jl,ipf)*debyeprof_d(2,jl,ipf)
              z1fsqz1sq_d= zfsq + z1sq_d
              z2fsqz2sq_d= zfsq + z2sq_d
              z1div_d    = 1.0/z1fsqz1sq_d
              z2div_d    = 1.0/z2fsqz2sq_d
              z1den_d    = z34dif_d*z1div_d
              z2den_d    = z45dif_d*z2div_d
              zastar_d   = debyeprof_d(3,jl,ipf) - zfsq*(z1den_d+z2den_d)
              z1prod_d   = z34dif_d*debyeprof_d(1,jl,ipf)
              z2prod_d   = z1prod_d*z1div_d
              z3prod_d   = z45dif_d*debyeprof_d(2,jl,ipf)
              z4prod_d   = z3prod_d*z2div_d
              zbstar_d   = -zf*(z2prod_d + z4prod_d)
              zbstarsq_d = zbstar_d*zbstar_d
              za2star_d  = zastar_d + 2.0
              za2starsq_d= za2star_d*za2star_d
              zdiv_d     = za2starsq_d + zbstarsq_d
              zgstar_d   = -3.0*zbstar_d/zdiv_d

              ! Now compute tangent-linear code

              z34dif     = debyeprof(3,jl,ipf) - debyeprof(4,jl,ipf)
              z45dif     = debyeprof(4,jl,ipf) - debyeprof(5,jl,ipf)
              z1sq       = 2.0*debyeprof_d(1,jl,ipf)*debyeprof(1,jl,ipf)
              z2sq       = 2.0*debyeprof_d(2,jl,ipf)*debyeprof(2,jl,ipf)
              z1fsqz1sq  = z1sq
              z2fsqz2sq  = z2sq
              z1div      = -z1fsqz1sq/(z1fsqz1sq_d*z1fsqz1sq_d)
              z2div      = -z2fsqz2sq/(z2fsqz2sq_d*z2fsqz2sq_d)
              z1den      = z34dif_d*z1div + z34dif*z1div_d
              z2den      = z45dif_d*z2div + z45dif*z2div_d
              zastar     = debyeprof(3,jl,ipf) - zfsq*(z1den+z2den)
              z1prod     = z34dif*debyeprof_d(1,jl,ipf)                 &
                           + z34dif_d*debyeprof(1,jl,ipf)
              z2prod     = z1prod*z1div_d + z1prod_d*z1div
              z3prod     = z45dif*debyeprof_d(2,jl,ipf)                 &
                           + z45dif_d*debyeprof(2,jl,ipf)
              z4prod     = z3prod*z2div_d + z3prod_d*z2div
              zbstar     = -zf*(z2prod + z4prod)
              zbstarsq   = 2.0*zbstar_d*zbstar
              za2star    = zastar
              za2starsq  = 2.0*za2star_d*za2star
              zdiv       = za2starsq + zbstarsq
              zgstar     = -3.0*(zdiv_d*zbstar - zdiv*zbstar_d)/(zdiv_d*zdiv_d)
              zopdp(jl,j)= zopdp(jl,j) - 1.5*zf*(zgstar*xxc_d(jl,ipf,1)  &
                           + zgstar_d*xxc(jl,ipf,1))   
        ENDIF
     ENDDO! End of level loop
  ENDDO! End of channel/profile loop

  where( zopdpref_d(:,:) > 0.0 )
     zopdp = 0.0
  end where

  DO jl = 1, jplev
     DO j = 1, knchpf
        ich = kchan(j)
        !     -----------------------------------------------------------------
        !          2.   Compute layer to space optical depths
        !               ------- ----- -- ----- ------- ------
        opdpa(j)   = opdpa(j)   + zopdp(jl,j)
        opdp(j,jl) = opdp(j,jl) + opdpa(j)
        !     -----------------------------------------------------------------
        !*         3.   Apply gamma corrections and change to +ve opt depth.
        !               ----- ----- ----------- --- ------ -- --- --- ------
        opdp(j,jl) = -opdp(j,jl)*ChanConstants(ksat) % gamma(ich)
        !     ------------------------------------------------------------------
        !          4.  Check for reasonable value
        !              ----- --- ---------- -----
        IF(jl == 1) THEN
           IF(opdpref_d(j,jl) < 0.) opdp(j,jl)=0.
        ELSE
           IF(opdpref_d(j,jl) < opdp_d(j,jl)) THEN
              opdp(j,jl) = opdp(j,jl-1)
           ENDIF
        ENDIF
        !
     ENDDO! End of channel/profile loop
  ENDDO! End of level loop
  ! 
  RETURN
END SUBROUTINE OPDEPTL
