!+ Fast radiative transfer model.
!
SUBROUTINE RTTOV &
     (knpf, klenpf, ppres, pangl, pangs, ksurf, ksat, knchpf,            &
     kchan, kprof, pav, psav, pssv, pcv, pemis, ifail, prad, ptb, radov, &
     rado, tau, tausfc, lcloud)
!
!    This software was developed within the context of 
!    the EUMETSAT Satellite Application Facility on 
!    Numerical Weather Prediction (NWP SAF), under the 
!    Cooperation Agreement dated 25 November 1998, between 
!    EUMETSAT and the Met Office, UK, by one or more partners 
!    within the NWP SAF. The partners in the NWP SAF are 
!    the Met Office, ECMWF, KNMI and MeteoFrance.
! 
!    Copyright 2002, EUMETSAT, All Rights Reserved.
!
! Description:
! to compute multi-channel level to space transmittances,
! top of atmosphere radiances and brightness
! temperatures for many profiles and Ir/Mw sensors. 
!
! Method
! see Matricardi et al QJ 2002 
! see Saunders et al QJ 1999 
! see: ECMWF Technical Memoranda 176/282/345 (Available from ECMWF)
!
! Current Code Owner: SAF NWP
!
! History:
! Version   Date        Comment
! -------   ----        -------
!          13/8/92.  For version 2.
!                    ksat added to argument list; ssu included;
!                    internal changes to move big arrays from commons to
!                    arguments and to introduce taskcommons
!           8/7/97   added ozone and extended water vapour in control vector
!        01/05/2000     F90 code 
!        21/08/2000  Interface to rtint changed to include pref (surface reflectivity).
!                    (Stephen English)
!        31/01/2001  More cloud computations. stored in radov (F. Chevallier)
!        6/2/2001    pgrody and knav etc arrays removed from call (R Saunders)
!        18/01/2002  Thread safe (D.Salmond)
!        03/12/2002  Added return if unphysical profile input (R. Saunders)
!        11/03/2004  Fix OPDEP AD,TL,K (A Collard)
!
! Code Description:
!   Language:          Fortran 90.
!   Software Standards: "European Standards for Writing and
!     Documenting Exchangeable Fortran 90 Code".
!     31/01/2001  More cloud computations. stored in radov (F. Chevallier)
! Declarations:
! Modules used:
!
  USE MOD_CPARAM, ONLY : &
  ! Imported Scalar Variables with intent (in):
  jppf     ,&   ! Max no. profiles
  jplev  ,&     ! No. of pressure levels
  jpchpf ,&     ! Max no. of profs * chans used
  njpnsat  ,&   ! Total max sats to be used
  njplev   ,&   ! No. of pressure levels
  njpnav   ,&   ! No. of profile variables
  njpnsav  ,&   ! No. of surface air variables
  njpnssv  ,&   ! No. of skin variables
  njpncv   ,&   ! No. of cloud variables
  njpcofm  ,&   ! Mixed gas coeffs (max)
  njpcofw  ,&   ! Water vapour coeffs (max)
  njpcofo       ! Ozone coeffs (max)
  !
  ! DEPENDS ON: PRFIN.o
  ! DEPENDS ON: OPDEP.o
  ! DEPENDS ON: RTTAU.o
  ! DEPENDS ON: RTINT.o
  IMPLICIT NONE
  ! Subroutine arguments
  ! Scalar arguments with intent(in):
  INTEGER , INTENT(in) :: knpf           ! Number of profiles
  INTEGER , INTENT(in) :: klenpf         ! Length of input  profile vectors 
  INTEGER , INTENT(in) :: ksat           ! Satellite index (see rttvi)
  INTEGER , INTENT(in) :: knchpf         ! Number of output radiances
                                         !    (= channels used * profiles) 
  LOGICAL, INTENT(in) :: lcloud          ! switch for cloud computations

  ! Array  arguments with intent(in):
  INTEGER , INTENT(in) :: kchan(knchpf)  ! Channel indices
  INTEGER , INTENT(in) :: kprof(knchpf)  ! Profiles indices
  INTEGER , INTENT(in) :: ksurf(knpf)    ! Surface type index
  REAL , INTENT(in)    :: ppres(njplev)  ! Pressure levels (hpa) of 
                                         !   atmospheric profile vectors
  REAL , INTENT(in)    :: pangl(knpf)    ! Satellite local zenith angle (deg)
  REAL , INTENT(in)    :: pangs(knpf)    ! Solar zenith angle at surface (deg)
  REAL , INTENT(in)    :: pav(njplev,njpnav,knpf)! Atmosp. profile variables
  REAL , INTENT(in)    :: psav(njpnsav,knpf)    ! Surface air variables
  REAL , INTENT(in)    :: pssv(njpnssv,knpf)    ! Surface skin variables
  REAL , INTENT(in)    :: pcv(njpncv,knpf)      ! Cloud variables

  ! Array  arguments with intent(inout):
  REAL , INTENT(inout) :: pemis(knchpf)         !  surface emissivities

  ! Scalar arguments with intent(out):

  ! Array  arguments with intent(out):
  INTEGER , INTENT(out) :: ifail(knpf,njpnsat)  !  return flag 
              !     0 = input profile OK
              ! 11-19 = outside profile limits
              !    11 =  temp profile
              !    12 =  specific humidity profile
              !    13 =  ozone profile
              !    14 =  surface temp profile
              !    15 =  surface specific humidity profile
              !    16 =  surface wind 
              ! 20-29 = unphysical profile 
              !    20 =  input pressure levels wrong
              !    21 =  temp profile
              !    22 =  specific humidity profile
              !    23 =  ozone profile
              !    24 =  surface temp profile
              !    25 =  surface specific humidity profile
              !    26 =  surface wind 
              !    27 =  surface pressure

  REAL , INTENT(out)    :: prad(knchpf)         ! radiances (mw/cm-1/ster/sq.m)
  REAL , INTENT(out)    :: ptb(knchpf)          ! brightness temperatures (K)
  REAL , INTENT(out)    :: rado(knchpf)         ! overcast radiance at given
                                                !   cloud top in mw/m2/sr/cm-1
  REAL , INTENT(out)    :: radov(knchpf,2*njplev+2) 
    ! RT quantities for possible cloud computations outside RTTOV, 
    !   in mw/m2/sr/cm-1 :
    ! radov (:,1:njplev)          : overcast radiance at given cloud top 
    ! radov (:,njplev+1,2*njplev) : contribution to radiance of
    !                               downward cloud emission at given cloud top 
    ! radov (:,2*njplev+1)        : clear-sky radiance without reflection term
    ! radov (:,2*njplev+2)        : reflected clear-sky downwelling radiance 

  REAL , INTENT(out)    :: tau(knchpf,njplev)  ! transmittance from each 
                                                !   standard pressure level
  REAL , INTENT(out)    :: tausfc(knchpf)      ! transmittance from surface

! Local arrays:

 REAL :: xpath(jppf)  ! Secant of viewing path angle at surface
 REAL :: xpath1(jppf) ! XPATH - 1
 REAL :: sqtpth(jppf) ! Sqrt of XPATH
 REAL :: xpaths(jppf) ! Secant of solar zenith angle at surface
 REAL :: snad2(jppf)  ! Sine nadir angle squared
 REAL :: cnad2(jppf)  ! Cos nadir angle squared
 REAL :: czen(jppf)   ! Cosine zenith angle
 REAL :: czen2(jppf)  ! Cos zenith angle squared
 REAL :: szen2(jppf)  ! Sine zenith angle squared
 INTEGER :: nstype(jppf)   ! Surface type index; 1=sea, 0=land 2=ice
 INTEGER :: nlevsf(jppf)   ! Index of nearest std press level at/below surface
 REAL    :: fracps(jppf)   ! Fraction of std press level interval by which
                                ! surface is above level NLEVSF.
 REAL    :: surfw(jppf)    ! Surface windspeed m/s
 INTEGER :: nlevcd(jppf)   ! Index of nearest std press level at/below
                            ! cloud top.
 REAL    :: fracpc(jppf)   ! Fraction of std press level interval by which
                            ! cloud is above level NLEVCD.
 REAL :: ts(jppf)          ! Surface skin temperature in K.
 REAL :: temp(jplev,jppf)  ! Temperature profile in K.
 REAL :: wmix(jplev,jppf)  ! Specific humidity profile in ppmv
 REAL :: ozon(jplev,jppf)  ! Ozone profile in ppmv
 REAL :: cldw(jplev,jppf)  ! Cloud liquid water in kg/kg
 REAL :: ta(jppf)          ! Surface air temperature in K.
 REAL :: wmixs(jppf)       ! Surface specific humidity in ppmv
 REAL :: surfp(jppf)       ! Surface pressure in mb=hPa.
 REAL :: cldp(jppf)        ! Cloud-top pressure in mb=hPa.
 REAL :: cldf(jppf)        ! Fractional (ir) cloud cover.
 REAL :: emis(jpchpf)      ! Surface emissivity
 REAL :: emis1(jpchpf)      ! Surface emissivity

  REAL :: opdp(knchpf,njplev)        ! Optical depth from press level to space 
  REAL :: opdpref(knchpf,njplev)     ! same as OPDP but without control check
  REAL :: zopdpref(njplev,knchpf)    ! same as OPDP but without control check
  REAL :: xxm(njplev,knpf,njpcofm)   ! Functions of profile for mixed gas 
  REAL :: xxw(njplev,knpf,njpcofw)   ! Functions of profile for water vapour 
  REAL :: xxo(njplev,knpf,njpcofo)   ! Functions of profile for ozone 
  REAL :: xxc(njplev,knpf,1)         ! Funct. of profile for cloud liquid water

  REAL :: b(knchpf,njplev)           ! PLANCK functions for temp. profiles
  REAL :: ba(knchpf)                 ! PLANCK functions for surface air temp.
  REAL :: bs(knchpf)                 ! PLANCK functions for surface skin temp.
  REAL :: bdt(knchpf,njplev)         ! Stores upwelling radiation from
                                     !   atmosphere above each level.
  REAL :: bdtr(knchpf,njplev)        ! Stores downwelling radiation at
                                     !   each level from atmosphere above
  REAL :: bdtrc(knchpf,njplev)       ! Stores downwelling radiation 
                                     !   below overcast cloud
  REAL :: fcld(knchpf)               ! Effective fractional cloud 
                                     !   cover for each chan * prof
  REAL :: radcl(knchpf)              ! Clear column radiances
  REAL :: tbcl(knchpf)               ! Clear column brightness temp. (K)
      
  REAL :: debyeprof(5,njplev,knpf)   ! Functions of debye terms of 
                                     !   profile for cloud calc
  REAL :: debyesv(5,njplev,knpf)     ! Functions of debye terms of 
                                     !   surface t for emiss calc
  REAL :: pref(knchpf)               ! reflectivities
  INTEGER  :: IPF                    ! profile counter
!- End of header ------------------------------------------------------

  !  The terms "constant" and "variable" are employed here in the sense used 
  !  in variational analysis, i.e. an input variable is a parameter with
  !  respect to which a gradient will be calculated in the associated
  !  tangent linear (TL) and adjoint (AD) routines. 
  !
  !
  !
  !-----------------------------------------------------------------
  !*         1.   Set up profile variables.
  !               --- -- ------- ---------
  CALL prfin(ppres, pav, psav, pssv, pcv, pemis, knpf, klenpf, &
       & knchpf, pangl, pangs, ksurf, ksat,     &
       & xxm, xxw, xxo, xxc, debyeprof, debyesv, &
       & nstype,&
       & xpath,xpath1,sqtpth,xpaths,snad2,cnad2,czen,czen2,szen2,&
       & nlevsf,fracps,surfw,nlevcd,fracpc,&
       & ts,temp,wmix,ozon,cldw,emis,ta,wmixs,surfp,cldp,cldf,&
       & ifail)
  ! unphysical profile return
  DO IPF = 1 , KNPF
    IF(ifail(ipf,ksat) .ge. 20 .and. ifail(ipf,ksat) .le. 27)RETURN
  ENDDO  
  !
  !-----------------------------------------------------------------
  !*         2.   Calculate optical depths.
  !               --------- ------- ------
  CALL opdep(knpf, kchan, kprof, knchpf, ksat,&
       & opdp, opdpref, zopdpref, &
       & xxm, xxw, xxo, xxc, debyeprof)
  !
  !-----------------------------------------------------------------
  !*         3.   Calculate transmittances on *rt* model levels.
  !               --------- -------------- -- ---- ----- ------
  CALL rttau(kprof, knchpf, opdp, tau, tausfc,nlevsf,fracps)
  !
  !-----------------------------------------------------------------
  !*         4.   Integrate *rt* equation.
  !               --------- ---- --------
  CALL rtint(kchan, kprof, knchpf, lcloud, prad, ptb,&
       & ksat, pemis, pref, tau, tausfc, b, ba, bs,&
       & radov, rado, bdt, bdtrc, bdtr, fcld, radcl, tbcl,&
       & nstype,nlevsf,fracps,&
       & xpath,snad2,cnad2,czen,czen2,szen2,surfw,nlevcd,fracpc,&
       & temp,ta,ts,cldf,emis,emis1)
  !
  !-----------------------------------------------------------------
  !
  RETURN
END SUBROUTINE RTTOV
